/****************************************************************************
** Meta object code from reading C++ file 'TodoModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../komodo-1.6.0/src/models/TodoModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TodoModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9TodoModelE_t {};
} // unnamed namespace

template <> constexpr inline auto TodoModel::qt_create_metaobjectdata<qt_meta_tag_ZN9TodoModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TodoModel",
        "QML.Element",
        "auto",
        "filePathChanged",
        "",
        "fileChanged",
        "fileModified",
        "addTodo",
        "QUuid",
        "description",
        "deleteTodo",
        "QModelIndex",
        "index",
        "setLocalFilePath",
        "localFile",
        "loadFile",
        "saveFile",
        "fileExists",
        "indexFromQUuid",
        "uuid",
        "disallowedKeyName",
        "keyName",
        "filePath",
        "QUrl",
        "startupSearchText",
        "fileNameArg",
        "Roles",
        "DescriptionRole",
        "CompletionRole",
        "PriorityRole",
        "CompletionDateRole",
        "CreationDateRole",
        "ContextsRole",
        "ProjectsRole",
        "KeyValuePairsRole",
        "PrettyDescriptionRole",
        "DueDateRole",
        "UUIDRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'filePathChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fileChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileModified'
        QtMocHelpers::SlotData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'addTodo'
        QtMocHelpers::MethodData<QUuid(const QString &)>(7, 4, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 9 },
        }}),
        // Method 'deleteTodo'
        QtMocHelpers::MethodData<void(const QModelIndex &)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Method 'setLocalFilePath'
        QtMocHelpers::MethodData<void(const QString &)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Method 'loadFile'
        QtMocHelpers::MethodData<bool()>(15, 4, QMC::AccessPublic, QMetaType::Bool),
        // Method 'saveFile'
        QtMocHelpers::MethodData<bool()>(16, 4, QMC::AccessPublic, QMetaType::Bool),
        // Method 'fileExists'
        QtMocHelpers::MethodData<bool() const>(17, 4, QMC::AccessPublic, QMetaType::Bool),
        // Method 'indexFromQUuid'
        QtMocHelpers::MethodData<QModelIndex(const QUuid &) const>(18, 4, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 8, 19 },
        }}),
        // Method 'disallowedKeyName'
        QtMocHelpers::MethodData<bool(const QString &) const>(20, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'filePath'
        QtMocHelpers::PropertyData<QUrl>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'startupSearchText'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'fileNameArg'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Roles'
        QtMocHelpers::EnumData<enum Roles>(26, 26, QMC::EnumFlags{}).add({
            {   27, Roles::DescriptionRole },
            {   28, Roles::CompletionRole },
            {   29, Roles::PriorityRole },
            {   30, Roles::CompletionDateRole },
            {   31, Roles::CreationDateRole },
            {   32, Roles::ContextsRole },
            {   33, Roles::ProjectsRole },
            {   34, Roles::KeyValuePairsRole },
            {   35, Roles::PrettyDescriptionRole },
            {   36, Roles::DueDateRole },
            {   37, Roles::UUIDRole },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<TodoModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject TodoModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9TodoModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9TodoModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9TodoModelE_t>.metaTypes,
    nullptr
} };

void TodoModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TodoModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->filePathChanged(); break;
        case 1: _t->fileChanged(); break;
        case 2: _t->fileModified(); break;
        case 3: { QUuid _r = _t->addTodo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUuid*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->deleteTodo((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 5: _t->setLocalFilePath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: { bool _r = _t->loadFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->saveFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->fileExists();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { QModelIndex _r = _t->indexFromQUuid((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->disallowedKeyName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TodoModel::*)()>(_a, &TodoModel::filePathChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TodoModel::*)()>(_a, &TodoModel::fileChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->filePath(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->startupSearchText(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->fileNameArg(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFilePath(*reinterpret_cast<QUrl*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *TodoModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TodoModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9TodoModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int TodoModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void TodoModel::filePathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TodoModel::fileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
